/*************************************************************************
 *
 *    (c) Olimex Ltd. 2010
 *
 *    File name   : BSP.c
 *    Description : Board support package
 *
 *    History :
 *    1. Date        : 25 May 2010
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/
#include "includes.h"

/*************************************************************************
 * Function Name: InitClock
 * Parameters: none
 *
 * Return: none
 *
 * Description: Init MCU clock
 *
 *************************************************************************/
void InitClock (void)
{
  // 1. Clocking the controller from internal HSI RC (8 MHz)
  RCC_HSICmd(ENABLE);
  // wait until the HSI is ready
  while(RCC_GetFlagStatus(RCC_FLAG_HSIRDY) == RESET);
  RCC_SYSCLKConfig(RCC_SYSCLKSource_HSI);
  // 2. Enable ext. high frequency OSC
  RCC_HSEConfig(RCC_HSE_ON);
  // wait until the HSE is ready
  while(RCC_GetFlagStatus(RCC_FLAG_HSERDY) == RESET);
  // 3. Init PLL
  RCC_PLLConfig(RCC_PLLSource_HSE_Div1,RCC_PLLMul_9); // 72MHz
  RCC_PLLCmd(ENABLE);
  // wait until the PLL is ready
  while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);
  // 4. Set system clock dividers
  RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_1Div5);
  RCC_PCLK2Config(RCC_HCLK_Div1);
  RCC_PCLK1Config(RCC_HCLK_Div2);
  RCC_HCLKConfig(RCC_SYSCLK_Div1);
#ifdef EMB_FLASH
  // 5. Init Embedded Flash
  // Zero wait state, if 0 < HCLK 24 MHz
  // One wait state, if 24 MHz < HCLK 56 MHz
  // Two wait states, if 56 MHz < HCLK 72 MHz
  // Flash wait state
  FLASH_SetLatency(FLASH_Latency_2);
  // Half cycle access
  FLASH_HalfCycleAccessCmd(FLASH_HalfCycleAccess_Disable);
  // Prefetch buffer
  FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);
#endif // EMB_FLASH
  // 5. Clock system from PLL
  RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
}

/*************************************************************************
 * Function Name: InitUserPeriph
 * Parameters: none
 *
 * Return: none
 *
 * Description: initializes pins used by the LEDS, USB connect input, wake-up button input
 *
 *************************************************************************/
void InitUserPeriph(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
	
  // Enable GPIO clocks
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC|RCC_APB2Periph_GPIOA, ENABLE);
  // Release GPIO reset
  RCC_APB2PeriphResetCmd(RCC_APB2Periph_GPIOC|RCC_APB2Periph_GPIOA, DISABLE);

  // STAT LED (PC13, PC14)
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13|GPIO_Pin_14;
  GPIO_Init(GPIOC,&GPIO_InitStructure);

  GPIO_WriteBit(GPIOC, GPIO_Pin_13, Bit_RESET);
  GPIO_WriteBit(GPIOC, GPIO_Pin_14, Bit_RESET);

  // USB connect pin (PC10)
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  GPIO_Init(GPIOC,&GPIO_InitStructure);

  // Power jack connect pin (PC12)
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
  GPIO_Init(GPIOC,&GPIO_InitStructure);

  // wakeup button pin (PA0)
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_Init(GPIOA,&GPIO_InitStructure);
}

/*************************************************************************
 * Function Name: StatLEDSet
 * Parameters: ledno - index of the led to alter
 *			enabled - TRUE to turn on, FALSE to turn off
 *
 * Return: none
 *
 * Description: turns on/off LED
 *
 *************************************************************************/
void StatLEDSet(int ledno, Boolean enabled)
{
	switch(ledno) {
	case 0:
		GPIO_WriteBit(GPIOC, GPIO_Pin_13, enabled ? Bit_SET : Bit_RESET);
		break;
	case 1:
		GPIO_WriteBit(GPIOC, GPIO_Pin_14, enabled ? Bit_SET : Bit_RESET);
		break;
	}
}

/*************************************************************************
 * Function Name: USBConnectedPinGet
 * Parameters: none
 *
 * Return: TRUE if cable is connected, FALSE if not
 *
 * Description: Returns a flag indicating whether a USB cable is connected
 *
 *************************************************************************/
Boolean USBConnectedPinGet(void)
{
	if( GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_10) )
		return TRUE;
	else
		return FALSE;
}

/*************************************************************************
 * Function Name: PowerJackConnectedPinGet
 * Parameters: none
 *
 * Return: TRUE if power cable jack is connected, FALSE if not
 *
 * Description: Returns a flag indicating whether a power jack cable is connected
 *
 *************************************************************************/
Boolean PowerJackConnectedPinGet(void)
{
	if( GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_12) )
		return TRUE;
	else
		return FALSE;	
}

/*************************************************************************
 * Function Name: USBConnectedPinGet
 * Parameters: none
 *
 * Return: TRUE if cable is pressed, FALSE if not
 *
 * Description: Returns a flag indicating whether a USB cable is connected
 *
 *************************************************************************/
Boolean WakeupButtonGet(Boolean relaxEnable)
{
#define WAKEUP_BTN_RELAX_COUNTER 100
	static int relaxCounter = WAKEUP_BTN_RELAX_COUNTER;
	
	if( GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_0) ) {
		if(relaxEnable) {
			if(relaxCounter) {
				if(--relaxCounter)
					return FALSE;
				else
					return TRUE;
			}
		}
		else
			return TRUE;
	}
	else {
		relaxCounter = WAKEUP_BTN_RELAX_COUNTER;
		return FALSE;	
	}
	
	return FALSE;
}

/*******************************************************************************
* Function Name  : ResetDevice
* Description    : Software reset
* Input          : Node
* Output         : None
* Return         : There is no return here!
*******************************************************************************/
void ResetDevice(void)
{
	Dly100us((void*)100);
	NVIC_GenerateSystemReset();
	__DSB();		/* Ensure completion of memory access */
	while(1);		/* wait until reset */
}

/*************************************************************************
 * Function Name: Dly100us
 * Parameters: void *arg
 *
 * Return: none
 *
 * Description: Delay 100us * arg
 *
 *************************************************************************/
void Dly100us(void *arg)
{
 Int32U Dly = (Int32U)arg;
  while(Dly--)
  {
    for(volatile int i = LOOP_DLY_100US; i; i--);
  }
}
