/*************************************************************************
 *
 *    (c) Olimex Ltd. 2010
 *
 *    File name   : BattMeas.c
 *    Description : Functions for measuring battery voltage.
 *
 *    Synopsis: This is nonblocking implementation of FSM for measurnig voltage with the ADC module.
 *              Vbat is measured at the middle point of a resistor divider circuit, the one end of which is
 *			 connected to Vbat and the other to an enable pin. The latter is done to save power between samples. In
 *			 oprder to measure voltage one should enable the lower end of the divider circuit, wait a bit and
 *			 start a conversion with the ADC input connected to the divider selected. Then Vref (which should be 1.2V)
 * 			 is measured the same way. Vbat then is calculated by the following formula:
 *				Vbat = 1.524 * Vmeasured / Vref, in Volts.
 *
 *    History :
 *    1. Date        : 25 May 2010
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/
#include "includes.h"


/* Private typedef -----------------------------------------------------------*/
typedef enum _BATTMEAS_STATE {
	BATTMEAS_STATE_IDLE = 0,
	BATTMEAS_STATE_WAIT_BATT,
	BATTMEAS_STATE_CONVERT_BATT,
	BATTMEAS_STATE_WAIT_REF,
	BATTMEAS_STATE_CONVERT_REF,
	BATTMEAS_STATE_CALCULATE
} BATTMEAS_STATE;

/* Private define ------------------------------------------------------------*/
#define BATTMEAS_WAIT_BETWEEN_CONVERSIONS 1000
#define BATTMEAS_WAIT_SETTLE_CHANNEL 1000

#define BATTMEAS_NUM_OF_SAMPLES 5

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static BATTMEAS_STATE battMeasState = BATTMEAS_STATE_IDLE;
static float voltage;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
static void BattMeasEnableDivider(Boolean enable);

/* External function prototypes -----------------------------------------------*/
// NOTE: It is not recommended to put anything here!!!

/*************************************************************************
 * Function Name: BattMeasInit
 * Parameters: none
 *
 * Return: none
 *
 * Description: Initialize ADC channel 10 and enable pin
 *
 *************************************************************************/
void BattMeasInit(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	ADC_InitTypeDef   ADC_InitStructure;
	
	// setup TIM2 to generate periodic interrupts
	{
		NVIC_InitTypeDef NVIC_InitStructure;
		TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;
		
		/* TIM2 clock enable */
		RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
		
		/* Enable the TIM2 gloabal Interrupt */
		NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQChannel;
		NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
		NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
		NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
		
		NVIC_Init(&NVIC_InitStructure);
		
		/* Time base configuration */
		TIM_TimeBaseStructure.TIM_Period = 65535;
		TIM_TimeBaseStructure.TIM_Prescaler = 500;
		TIM_TimeBaseStructure.TIM_ClockDivision = 0;
		TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;		
		TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
		
		/* TIM IT enable */
		TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);
				
		/* TIM2 enable counter */
		TIM_Cmd(TIM2, ENABLE);		
	}
	
	// ADC init
	// ADC Deinit
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
	ADC_DeInit(ADC1);
	
	BattMeasEnableDivider(TRUE);
	
	// analog input (RC0/ADC10)
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Speed = (GPIOSpeed_TypeDef)0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	GPIO_Init (GPIOC, &GPIO_InitStructure);
	
	// ADC Structure Initialization
	ADC_StructInit(&ADC_InitStructure);
	
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;
	ADC_Init(ADC1, &ADC_InitStructure);
	
	// Enable the ADC
	ADC_Cmd(ADC1, ENABLE);
	// Enable Vrefint channel17
	ADC_TempSensorVrefintCmd(ENABLE);

	
	// ADC calibration
	// Enable ADC1 reset calibration register
	ADC_ResetCalibration(ADC1);
	// Check the end of ADC1 reset calibration register
	while(ADC_GetResetCalibrationStatus(ADC1) == SET);
	
	// Start ADC1 calibration
	ADC_StartCalibration(ADC1);
	// Check the end of ADC1 calibration
	while(ADC_GetCalibrationStatus(ADC1) == SET);
	
	// Configure channel
	ADC_RegularChannelConfig(ADC1, ADC_Channel_10, 1, ADC_SampleTime_239Cycles5);
	
	battMeasState = BATTMEAS_STATE_IDLE;
	voltage = 4.0; // this is the startup voltage of battery, loaded to prevent low battery at startup
}

/*************************************************************************
 * Function Name: BattMeasDoWork(..)
 * Parameters: none
 *
 * Return: none
 *
 * Description:
 *
 *************************************************************************/
void BattMeasDoWork(void)
{	
	static int waitCounter = BATTMEAS_WAIT_BETWEEN_CONVERSIONS;
	static int samplesCounter;
	static int samplesBatt, samplesRef;
	
	switch(battMeasState) {
	case BATTMEAS_STATE_IDLE:
		if(!waitCounter--) {
//			BattMeasEnableDivider(TRUE);
			ADC_RegularChannelConfig(ADC1, ADC_Channel_10, 1, ADC_SampleTime_239Cycles5);			
			
			waitCounter = BATTMEAS_WAIT_SETTLE_CHANNEL;
			battMeasState = BATTMEAS_STATE_WAIT_BATT;
		}
		break;

	case BATTMEAS_STATE_WAIT_BATT:
		if(!waitCounter--) {
			samplesCounter = 0;
			ADC_SoftwareStartConvCmd(ADC1, ENABLE); // Start conversion
			
			battMeasState = BATTMEAS_STATE_CONVERT_BATT;
		}
		break;
		
	case BATTMEAS_STATE_CONVERT_BATT:
		if(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == SET) {
			if(samplesCounter == 0)
				samplesBatt = ADC_GetConversionValue(ADC1);
			else
				samplesBatt += ADC_GetConversionValue(ADC1);
				
			if(++samplesCounter >= BATTMEAS_NUM_OF_SAMPLES) {
//				BattMeasEnableDivider(FALSE);
				ADC_RegularChannelConfig(ADC1, ADC_Channel_17, 1, ADC_SampleTime_239Cycles5);
				
				
				waitCounter = BATTMEAS_WAIT_SETTLE_CHANNEL;
				battMeasState = BATTMEAS_STATE_WAIT_REF;			
			}
			else {
				ADC_SoftwareStartConvCmd(ADC1, ENABLE); // Start conversion
			}
		}
		break;
	
	case BATTMEAS_STATE_WAIT_REF:
		if(!waitCounter--) {
			samplesCounter = 0;
			ADC_SoftwareStartConvCmd(ADC1, ENABLE); // Start conversion
			
			battMeasState = BATTMEAS_STATE_CONVERT_REF;
		}
		break;
		
	case BATTMEAS_STATE_CONVERT_REF:
		if(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == SET) {
			if(samplesCounter == 0)
				samplesRef = ADC_GetConversionValue(ADC1);
			else
				samplesRef += ADC_GetConversionValue(ADC1);
				
			if(++samplesCounter >= BATTMEAS_NUM_OF_SAMPLES) {
				battMeasState = BATTMEAS_STATE_CALCULATE;
			}
			else {
				ADC_SoftwareStartConvCmd(ADC1, ENABLE); // Start conversion
			}
		}
		break;
	case BATTMEAS_STATE_CALCULATE:
		voltage = (float)samplesBatt;
		
		voltage *= 1.524;
		voltage /= (float)samplesRef;

		waitCounter = BATTMEAS_WAIT_BETWEEN_CONVERSIONS;
		battMeasState = BATTMEAS_STATE_IDLE;
		break;
		
	default:
		break;
	}
}

/*************************************************************************
 * Function Name: BattMeasGetVoltage(..)
 * Parameters: none
 *
 * Return: none
 *
 * Description:
 *
 *************************************************************************/
float BattMeasGetVoltage(void)
{
	return voltage;
}

void Tim2PeriodicHandler(void)
{
	static int i = 0;
	
	// if powered by the battery and battery voltage is below 3.3V then start blinking the LED
	if( (!USBConnectedPinGet()) && (!PowerJackConnectedPinGet()) &&  (BattMeasGetVoltage() < 3.3)) {
		StatLEDSet(1, ((++i & 0x3) == 0x3) ? TRUE : FALSE);
	} else if((USBConnectedPinGet() || PowerJackConnectedPinGet()) && (BattMeasGetVoltage() < 4.0)) {
		StatLEDSet(1, TRUE);
	} else{
		StatLEDSet(1, FALSE);
	}
}

/* LOCAL FUNCTIONS DEFINITIONS */

static void BattMeasEnableDivider(Boolean enable)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	// BAT_SENSE_E (PC1)
	if(enable) {
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
		GPIO_Init(GPIOC, &GPIO_InitStructure);		
		
		GPIO_WriteBit(GPIOC, GPIO_Pin_1, Bit_RESET);
	}
	else {
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
		GPIO_Init(GPIOC, &GPIO_InitStructure);		
	}
}



