/*************************************************************************
 *
 *    (c) Olimex Ltd. 2010
 *
 *    File name   : Mp3PlayerTask.c
 *    Description : implementation of the MP3 player
 *
 *    History :
 *    1. Date        : 03 May 2010
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/
#include "includes.h"

/* Private typedef -----------------------------------------------------------*/
typedef enum _MP3_PLAYER_STATE {
	MP3_STATE_IDLE = 0,
	MP3_STATE_INITIALIZING,
	MP3_STATE_PLAYING,
	MP3_STATE_PAUSED,
	MP3_STATE_STOPPED,
	MP3_STATE_ERROR
} MP3_PLAYER_STATE;

/* Private define ------------------------------------------------------------*/
#define DIR_NAME_MAX_LENGTH 256 // including terminator
#define ROOT_DIR_STRING "/"

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static EmbeddedFileSystem EFS;
static DirList list;
static File Mp3File;
static Mp3Stream_t Mp3Stream;
static Int8U Volume;

__no_init Int32U Mp3Buffer[512 / sizeof(Int32U)];

/* Private function prototypes -----------------------------------------------*/
void ConvertFilename(char* outStr, ListDirEntry *pFile);
/* Private functions ---------------------------------------------------------*/

/* External function prototypes -----------------------------------------------*/
// NOTE: It is not recommended to put anything here!!!


/*******************************************************************************
* Function Name  : __
* Description    : __
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Mp3PlayerTask(void)
{
	int currentState = MP3_STATE_INITIALIZING; // current and previous state in player FSM
	char errorMsg[32];
	char currDirName[DIR_NAME_MAX_LENGTH];
	int index;
	Boolean playingFlag = FALSE;
	Boolean enableCommands = TRUE;
	
	while(1) {
		// manage states here
		switch(currentState) {
		case MP3_STATE_IDLE:
			StatLEDSet(0, FALSE);
			break;

		case MP3_STATE_INITIALIZING:
			// Initialize UART user interface here (conf pins, conf baudrate etc.)
			UartUserInit();
			BattMeasInit();
			
			// initialize audio codec
			{
				UartUserAppendMsg("Starting...\r\n");

				MP3_Status_t MP3_Status;
				Mp3SendCmd(Mp3CmdPowerUp, (pInt32U)&MP3_Status);
				if(MP3_Status != MP3_Pass) {
					currentState = MP3_STATE_ERROR;
					strcpy(errorMsg, "\r\nError initializing audio codec!!\r\n");
					break;
				} else {
					UartUserAppendMsg("Codec initialized.\r\n");
				}
				
				Volume = 0x18;
				Int32U volTmp = Volume | (Volume << 8);
				Mp3SendCmd(Mp3CmdSetVol, &volTmp);
			}
			
			// initialize sd card and file system
			{
				// Init MMC/SD driver
				SdDiskInit();
		
				if( efs_init(&EFS, NULL) ) {
					currentState = MP3_STATE_ERROR;
					strcpy(errorMsg, "\r\nError initializing SD card!!\r\n");	
					break;
				} else {
					UartUserAppendMsg("SD card initialized.\r\n");
				}
				
				// open root folder
				strcpy(currDirName, "/");
				if( ls_openDir( &list, &(EFS.myFs) , currDirName) ) {
					strcpy(errorMsg, "\r\nError openning root dir!!\r\n");
					currentState = MP3_STATE_ERROR;
					break;
				} else {
					UartUserAppendMsg("Root dir found and opened.\r\n");
				}
				
				// send result from initialization
				UartUserSendMsg();
			}
			
			currentState = MP3_STATE_IDLE;
			continue;
			
		case MP3_STATE_PLAYING:
			playingFlag = TRUE;
			StatLEDSet(0, TRUE);
			int streamUnblocker;
			streamUnblocker = 20; // after this period loop will be forcefully broken
			while( MP3_DReq() && streamUnblocker--) {
				if((Mp3Stream.Size = file_read(&Mp3File, sizeof(Mp3Buffer), (pInt8U)Mp3Buffer)) != 0) {
					Mp3Stream.pStream = Mp3Buffer;
					Mp3Stream.PlaySpeed = Mp3PlayNorm;
					Mp3SendCmd(Mp3CmdPlay,(pInt32U)&Mp3Stream);
				}
				else {
					// whole file is read
					currentState = MP3_STATE_STOPPED;
					break;
				}
			}
			break;

		case MP3_STATE_PAUSED:
			{
				static Int32U counter = 0;
				
				counter++;
					
				StatLEDSet(0, counter & 0x2000);
			}
			break;
			
		case MP3_STATE_STOPPED:
			file_fclose(&Mp3File);
			{
				Int32U Tmp;
				Mp3SendCmd(Mp3CmdPlayStop, &Tmp);
			}
			playingFlag = FALSE;
			
			// mark stopped state
			while( !UartUserResetMsg() )
				;
			UartUserAppendMsg("\r\nStopped.\r\n");
			UartUserSendMsg();			
			
			currentState = MP3_STATE_IDLE;
			break;

		case MP3_STATE_ERROR:
			UartUserAppendMsg(errorMsg);
			UartUserSendMsg();			

			enableCommands = FALSE;

			currentState = MP3_STATE_IDLE;			
			break;

		default:
			break;
		}

		
		
		// check for pending commands here
		if(enableCommands) {
			switch( UartUserGetCurrentCommand() ) {
			case MP3_COMMAND_NONE:
				
				break;
				
			case MP3_COMMAND_LIST_DIR:
				// command is executable only if file is not playing
				if(playingFlag)
					break;
				
				while( !UartUserResetMsg() )
					;
				
				if( ls_openDir(&list, &(EFS.myFs) , currDirName) ) { // open current dir
					UartUserAppendMsg("\r\nError opening current directory!\r\nReturning to root.\r\n");
					strcpy(currDirName, ROOT_DIR_STRING);				
				}
				else {				
					// read all files and dirs
					int index = 0;
					
					while( !ls_getNext(&list) ) {
						char tempStr[LIST_MAXLENFILENAME];
						sprintf(tempStr, "\r\n%.2d. ", index++);
						UartUserAppendMsg(tempStr);
	
						ConvertFilename(tempStr, &list.currentEntry);
						UartUserAppendMsg(tempStr);
						
						// if file is a directory append tag
						if(list.currentEntry.Attribute & ATTR_DIRECTORY) {
							UartUserAppendMsg("\t\t<DIR>");
						}
						
						// send each file separately because list can become quite long
						UartUserSendMsg();
						while( !UartUserResetMsg() )
							UartUserDoWork();
					}
					
					// if directory is empty then display a message telling that
					if(!index) {
							UartUserAppendMsg("\r\nDirectory is empty!");
					}
				}
				UartUserAppendMsg("\r\n");
				UartUserSendMsg();
				break;
				
			case MP3_COMMAND_OPEN_DIR_NAME:
				// command is executable only if file is not playing
				if(playingFlag)
					break;
	
				while( !UartUserResetMsg() )
					;
	
				// check for correct number of parameters
				if(UartUserGetParamsNum() != 1) {
					UartUserAppendMsg("\r\nMissing/incorrect directory name!\r\n");
				}
				else {
					int strEnd = strlen(currDirName);
					strcat(currDirName, UartUserGetParam(0));
					strcat(currDirName, "/");
					if( ls_openDir(&list, &(EFS.myFs) , currDirName) ) { // open current dir
						currDirName[strEnd] = '\0'; // restore to previous
						UartUserAppendMsg("\r\nError opening new directory!\r\n");
					}
					else {
						char tmpStr[300];
						sprintf(tmpStr, "\r\nNew current dir:\r\n\t%s\r\n", currDirName);
						UartUserAppendMsg(tmpStr);
					}
				}
				UartUserSendMsg();
				break;
				
			case MP3_COMMAND_OPEN_DIR_INDEX:
				// command is executable only if file is not playing
				if(playingFlag)
					break;
				
				while( !UartUserResetMsg() )
					;
	
				// check for correct number of parameters
				if((UartUserGetParamsNum() != 1) || (sscanf(UartUserGetParam(0), "%d", &index) != 1) ) {
					UartUserAppendMsg("\r\nMissing/incorrect directory index!\r\n");
				}
				else {
					if( ls_openDir(&list, &(EFS.myFs) , currDirName) ) { // open current dir
						UartUserAppendMsg("\r\nError opening current directory!\r\nReturning to root.\r\n");
						strcpy(currDirName, ROOT_DIR_STRING);				
					}
					else {
						// search for a directory with given index
						while( !ls_getNext(&list) ) {
							if( (list.currentEntry.Attribute & ATTR_DIRECTORY) && (!index--) )
								break;
						}
							
						if(index >= 0) {
							UartUserAppendMsg("\r\nDirectory not found!\r\n");
						}
						else {
							char tempStr[LIST_MAXLENFILENAME];
							int strEnd = strlen(currDirName);
							
							// construct new directory path
							ConvertFilename(tempStr, &list.currentEntry);
							strcat(currDirName, tempStr);
							strcat(currDirName, "/");
	
							if( ls_openDir(&list, &(EFS.myFs) , currDirName) ) { // open current dir
								currDirName[strEnd] = '\0'; // restore to previous
								UartUserAppendMsg("\r\nError opening new directory!\r\n");
							}
							else {
								char tmpStr[300];
								sprintf(tmpStr, "\r\nNew current dir:\r\n\t%s\r\n", currDirName);
								UartUserAppendMsg(tmpStr);
							}
						}
					}
				}
				UartUserSendMsg();
				break;
	
			case MP3_COMMAND_DIR_UP:
				// command is executable only if file is not playing
				if(playingFlag)
					break;
	
				while( !UartUserResetMsg() )
					;
	
				if(strcmp(currDirName, "/") == 0) {
					UartUserAppendMsg("\r\nAlready in root dir!\r\n");
				}
				else {
					// strip off last dir in path
					char *p;
					p = strrchr(currDirName, '/');
					*p = '\0';
					p = strrchr(currDirName, '/');
					*(p + 1) = '\0';
					
					if( ls_openDir(&list, &(EFS.myFs) , currDirName) ) { // open current dir
						UartUserAppendMsg("\r\nError opening current directory!\n\rReturning to root.\r\n");
						strcpy(currDirName, ROOT_DIR_STRING);				
					}
					else {
						char tmpStr[300];
						sprintf(tmpStr, "\r\nNew current dir:\r\n\t%s\r\n", currDirName);
						UartUserAppendMsg(tmpStr);
					}
				}
				
				UartUserSendMsg();			
				break;
				
			case MP3_COMMAND_PLAY_FILE_NAME:
				// command is executable only if file is not playing
				if(playingFlag)
					break;
			
				while( !UartUserResetMsg() )
					;
	
				// check parameters
				if(	(UartUserGetParamsNum() != 1) || // we need exactly 1 parameter
					(strlen(UartUserGetParam(0)) < 5) || // filename should be at least 5 cahracters (x.MP3)
					(strcmp(UartUserGetParam(0) + strlen(UartUserGetParam(0)) - 4, ".MP3")) // file should have ext ".MP3" CASE-SENSITIVE!!
				  )
				{
					UartUserAppendMsg("\r\nMissing/incorrect filename name!\r\n");
				}
				else {
					if( ls_openDir(&list, &(EFS.myFs) , currDirName) ) { // open current dir
						UartUserAppendMsg("\r\nError opening current directory!\n\rReturning to root.\r\n");
						strcpy(currDirName, ROOT_DIR_STRING);				
					}
					else {
						char path[300];
						strcpy(path, currDirName);
						strcat(path, UartUserGetParam(0));
						
						// try to open the file
						if(file_fopen(&Mp3File,&(EFS.myFs), path, MODE_READ) != 0) {
							UartUserAppendMsg("\r\nCould not open file or file missing!\r\n");
						}
						else {
							// here file is now open.
							UartUserAppendMsg("\r\nStarted playing ");
							UartUserAppendMsg(path);
							UartUserAppendMsg("\r\n");
							currentState = MP3_STATE_PLAYING;
						}
					}
				}
	
				UartUserSendMsg();		
				break;
				
			case MP3_COMMAND_PLAY_FILE_INDEX:
				// command is executable only if file is not playing
				if(playingFlag)
					break;
	
				while( !UartUserResetMsg() )
					;
	
				// check parameters
				if((UartUserGetParamsNum() != 1) || (sscanf(UartUserGetParam(0), "%d", &index) != 1) ) {
					UartUserAppendMsg("\r\nMissing/incorrect file index!\r\n");
				}
				else {
					if( ls_openDir(&list, &(EFS.myFs) , currDirName) ) { // open current dir
						UartUserAppendMsg("\r\nError opening current directory!\n\rReturning to root.\r\n");
						strcpy(currDirName, ROOT_DIR_STRING);
					}
					else {
						char tempStr[LIST_MAXLENFILENAME];
						
						// search for a directory with given index
						while( !ls_getNext(&list) ) {						
							ConvertFilename(tempStr, &list.currentEntry);
							if( (!strcmp(tempStr + strlen(tempStr) - 4, ".MP3")) && (!index--) )
								break;
						}
							
						if(index >= 0) {
							UartUserAppendMsg("\r\nFile not found!\r\n");
						}
						else {
							char path[300];
							strcpy(path, currDirName);
							strcat(path, tempStr);
							
							if(file_fopen(&Mp3File, &(EFS.myFs), path, MODE_READ) != 0) {
								UartUserAppendMsg("\r\nCould not open file!\r\n");
							}
							else {
								// here file is now open.
								UartUserAppendMsg("\r\nStarted playing ");
								UartUserAppendMsg(path);
								UartUserAppendMsg("\r\n");
								currentState = MP3_STATE_PLAYING;
							}
						}
					}
				}
				
				UartUserSendMsg();
				break;
				
			case MP3_COMMAND_TURN_OFF_CODEC:
				{ // send command to codec
					Int32U Tmp;
					Mp3SendCmd(Mp3CmdPowerDown, &Tmp);
				}
				{ // message with the new volume value
					char tmpStr[16];
					sprintf(tmpStr, "\r\n%s\r\n", "Codec is OFF");
					UartUserAppendMsg(tmpStr);
					UartUserSendMsg();
				}
				break;
	
			case MP3_COMMAND_TURN_ON_CODEC:
				{ // send command to codec
					Int32U Tmp;
					Mp3SendCmd(Mp3CmdPowerUp, &Tmp);
				}
				{ // message with the new volume value
					char tmpStr[16];
					sprintf(tmpStr, "\r\n%s\r\n", "Codec is ON");
					UartUserAppendMsg(tmpStr);
					UartUserSendMsg();
				}
				break;
		
				
			// these are only executed during playback
			case MP3_COMMAND_STOP:
				if(playingFlag) {
					currentState = MP3_STATE_STOPPED;
				}
				
				break;
	
			case MP3_COMMAND_PLAY_PAUSE:
				if(playingFlag) {
					while( !UartUserResetMsg() )
						;
					
					if(currentState == MP3_STATE_PAUSED) {
						UartUserAppendMsg("\r\nPlaying\r\n");
						currentState = MP3_STATE_PLAYING;
					}
					else {
						UartUserAppendMsg("\r\nPaused\r\n");
						currentState = MP3_STATE_PAUSED;
					}
	
					UartUserSendMsg();
				}
				
				break;
	
			case MP3_COMMAND_VOL_UP:
				if(Volume > 0)
					Volume--;
				goto	volume_common;
				
			case MP3_COMMAND_VOL_DN:
				if(Volume < 0xFE)
					Volume++;
		volume_common:
				{ // send command to codec
					Int32U volTmp = Volume | (Volume << 8);
					Mp3SendCmd(Mp3CmdSetVol, &volTmp);
				}
				{ // message with the new volume value
					char volStr[16];
					sprintf(volStr, "\r\nVol: %d\r\n", Volume);
					UartUserAppendMsg(volStr);
					UartUserSendMsg();
				}
				
				break;
	
			case MP3_COMMAND_HELP:
				UartUserAppendMsg("\r\nSupported commands:\r\n"
							   "L - lists the current dir\r\n"
							   "CDIRNAME <name> - enters dir by name\r\n"
							   "cnum <index> - enters dir by index\r\n"
							   ". - returns to the parent dir\r\n"
							   "PSONGNAME <name> - plays file by name\r\n"
							   "pnum <num> - plays file by index\r\n"
							   "OFF - turns the unit off\r\n"
							   "ON - turns the unit on\r\n"
							   "C - stop playing current file\r\n"
							   "+ - increase volume by 1dB\r\n"
							   "- - decrease volume by 1dB\r\n"
							   "p - pause/play toggle\r\n"
							   "? - display this help\r\n");
				UartUserSendMsg();

				break;
	
			default:
				break;
			}
		}
		
		// enable recetion of new commands here
		UartUserDiscardCurrentCommand();
			
		// check for card present and issue error if needed
		SdStatusUpdate();
		pDiskCtrlBlk_t pSD_DiskStatus = SdGetDiskCtrlBkl();
		if(pSD_DiskStatus->DiskStatus != DiskCommandPass) {
			static Boolean beenHere = FALSE;			
			if(!beenHere) {
				beenHere = TRUE;
				currentState = MP3_STATE_ERROR;
				strcpy(errorMsg, "\r\nPls, Insert Card and restart!\r\n");
			}
		}
		
		/* call other subroutines here */
		UartUserDoWork();
		
		// check if USB cable is connected, if so RESET processor
		if( USBConnectedPinGet() ) {
			ResetDevice();
		}
		
		// wakeup button is used as a PLAY/PAUSE button
		if( enableCommands && WakeupButtonGet(TRUE) ) {
			UartUserGenerateCommand(MP3_COMMAND_PLAY_PAUSE, NULL, 0);
		}
		
		// routine for battery measuring
		BattMeasDoWork();
	}
}

void ConvertFilename(char* outStr, ListDirEntry *pFile)
{
	int i;
	
	if(list.currentEntry.Attribute & ATTR_DIRECTORY)
	{ // DIRECTORY
		for(i = 0; (pFile->FileName[i] != ' ') && (i < LIST_MAXLENFILENAME); i++) {
			*outStr++ = pFile->FileName[i];
		}
		*outStr = '\0';
	}
	else
	{ // FILE
		for(i = 0; (pFile->FileName[i] != ' ') && (i < 8); i++) {
			*outStr++ = pFile->FileName[i];
		}
		*outStr++ = '.';
		
		for(i = 8; i < 11; i++) {
			*outStr++ = pFile->FileName[i];
		}
					
		*outStr = '\0';
	}

}


