/*************************************************************************
 *
 *    (c) Olimex Ltd. 2010
 *
 *    File name   : uartUserInterface.c
 *    Description : Implementation of the user interface using USART2 module
 *
 *    History :
 *    1. Date        : 03 May 2010
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/
#include "includes.h"

/* Private typedef -----------------------------------------------------------*/
typedef struct _STRING_COMMAND_PAIR {
	char *commandStr;
	MP3_PLAYER_UART_COMMAND command;
} STRING_COMMAND_PAIR;

/* Private define ------------------------------------------------------------*/
#define PARAMS_MAX_NUM 5		// max number of parameters a command can accept
#define PARAM_CHAR_LENGTH 16	// max length in characters (+ null terminator) a param can have
#define RX_BUFFER_LENGTH 64	// length of the receive buffer
#define TX_BUFFER_LENGTH 512	// length of the transmit buffer

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static MP3_PLAYER_UART_COMMAND currentCommand; 				// the last reveived and recognised command
static char commandParams[PARAMS_MAX_NUM][PARAM_CHAR_LENGTH];	// parameters of the last reveived and recognised command

static char rxBuffer[RX_BUFFER_LENGTH], txBuffer[TX_BUFFER_LENGTH];	// rx and tx buffers
static int rxBufferIndex, txBufferIndex;						// pointer to the current position in the rx and tx buffers
static Boolean rxBufferLocked;								// set when '\n' is received to lock rx buffer and enable processing
static Boolean txBufferLocked;								// flag to lock tx buffer while it is being sent
	
static STRING_COMMAND_PAIR supportedCommands[] = {
	{ "L\n",			MP3_COMMAND_LIST_DIR },			// lists the current directory
	{ "CDIRNAME ",		MP3_COMMAND_OPEN_DIR_NAME },		// enters directory with a specific 8-character name
	{ "cnum ",		MP3_COMMAND_OPEN_DIR_INDEX },		// enters directory with a specific number, starting from 0
	{ ".\n",			MP3_COMMAND_DIR_UP },			// returns to the parent directory
	{ "PSONGNAME ",	MP3_COMMAND_PLAY_FILE_NAME },		// plays song with a specific name 8.3 encoded name
	{ "pnum ",		MP3_COMMAND_PLAY_FILE_INDEX },	// plays song with a specific number
	{ "OFF\n",		MP3_COMMAND_TURN_OFF_CODEC },		// turns the unit off
	{ "ON\n",			MP3_COMMAND_TURN_ON_CODEC },		// turns the unit on
	{ "C\n",			MP3_COMMAND_STOP },				// stop playing, returns 'Stopped.'
	{ "+\n",			MP3_COMMAND_VOL_UP },			// increase volume by 1dB, returns new volume setting (one byte)
	{ "-\n",			MP3_COMMAND_VOL_DN },			// decrease volume by 1dB, returns new volume setting (one byte)
	{ "p\n",			MP3_COMMAND_PLAY_PAUSE },		// pause/play toggle, returns 'Paused' for paused, 'Playing' for continue play
	{ "?\n",			MP3_COMMAND_HELP },				// display help with commands syntax
	
	{ "",			MP3_COMMAND_NONE }				// needed to determine end of commands list
};

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/*******************************************************************************
* Function Name  : UartUserInit(..)
* Description    : Initializes pins used by the USART2 module, configures module and enables interrupts
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void UartUserInit(void)
{	
	GPIO_InitTypeDef GPIO_InitStructure;
	USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;
	
	// init all used variables
	currentCommand = MP3_COMMAND_NONE;
	rxBufferIndex = 0;
	txBufferIndex = 0;
	rxBufferLocked = FALSE;
	txBufferLocked = FALSE;	
	
	/* Enable GPIO clock */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);
	
	/* Disable the USART2 Pins Software Remapping */
	GPIO_PinRemapConfig(GPIO_Remap_USART2, DISABLE);
	
	/* Enable UART clock */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);
	
	/* Configure USART Tx as alternate function push-pull */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	/* Configure USART Rx as input floating */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	/* USART configuration */
	USART_InitStructure.USART_BaudRate = 9600;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	USART_Init(USART2, &USART_InitStructure);
	
	/* Enable USART */
	USART_Cmd(USART2, ENABLE);
	
//	/* Enable the USART2 Transmit interrupt: this interrupt is generated when the
//		USART2 transmit data register is empty */
//	USART_ITConfig(USART2, USART_IT_TXE, ENABLE);
	
	/* Enable the USART2 Receive interrupt: this interrupt is generated when the
		USART2 receive data register is not empty */
	USART_ITConfig(USART2, USART_IT_RXNE, ENABLE);
	
	/* Enable the USARTx Interrupt */
	NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}

/*******************************************************************************
* Function Name  : UartUserDoWork(..)
* Description    : Call this function as often as possible, processes incomming commands
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void UartUserDoWork(void)
{
	static int commandIndex = 0;
	
	// check if received string matches any command string
	if( (currentCommand == MP3_COMMAND_NONE) && rxBufferLocked ) {
		// check if all commands processed
		if(supportedCommands[commandIndex].commandStr[0] == '\0') {
			rxBufferLocked = FALSE;
			commandIndex = 0;
			if( UartUserAppendMsg("\r\nCommand not recognised/supported!\r\n") )
				UartUserSendMsg();
		}
		else {
			// check for command string match
			if( strncmp(supportedCommands[commandIndex].commandStr,
					  rxBuffer,
					  strlen(supportedCommands[commandIndex].commandStr)) == 0) {
										
				// fill in parameters strings if any
				{
					int i, j;
					char *p, *pNew;

					// zero all parameters
					for(i = 0; i < PARAMS_MAX_NUM; i++)
						commandParams[i][0] = '\0';

					p = pNew = rxBuffer + strlen(supportedCommands[commandIndex].commandStr);
					for(i = 0; i < PARAMS_MAX_NUM; i++) {
						if( (pNew = strchr(p, (int)' ')) != NULL) {
							// copy param string
							for(j = 0; (p < pNew) && j < PARAM_CHAR_LENGTH - 1; j++)
								commandParams[i][j] = *p++;
							commandParams[i][j] = '\0';
							p++; // jump over space character
						}
						else {
							// copy LAST param string
							for(j = 0; (*p != '\n') && (*p != '\0'); j++)
								commandParams[i][j] = *p++;
							commandParams[i][j] = '\0';
							break;
						}
					}
				}
				
				// store new command
				currentCommand = supportedCommands[commandIndex].command;

				// enable reception of new data in the buffer
				rxBufferLocked = FALSE;
				commandIndex = 0;
			}
			else {
				// move on to the next command
				commandIndex++;
			}
		}
	}
	else {
		commandIndex = 0;
	}
			
    if(txBufferLocked && !txBufferIndex) {
	    if( USART_GetFlagStatus(USART2, USART_FLAG_TXE) == SET ) {
		    	/* Enable the USART2 Transmit interrupt: this interrupt is generated when the
				USART2 transmit data register is empty */
			USART_ITConfig(USART2, USART_IT_TXE, ENABLE);
	    }
    }
	
    // TODO: add other things to do here
}

/*******************************************************************************
* Function Name  : UartUserGetCurrentCommand(..)
* Description    : Gets the last received and recognized command
* Input          : None
* Output         : None
* Return         : The last received and recognized command
*******************************************************************************/
MP3_PLAYER_UART_COMMAND UartUserGetCurrentCommand(void)
{
	return currentCommand;
}

/*******************************************************************************
* Function Name  : UartUserDiscardCurrentCommand(..)
* Description    : Call this function after processing the current command to enable reception of the next
* Input          : None
* Output         : None
* Return         : The last received and recognized command
*******************************************************************************/
void UartUserDiscardCurrentCommand(void)
{
	currentCommand = MP3_COMMAND_NONE;
}

/*******************************************************************************
* Function Name  : UartUserGetParam(..)
* Description    : Gets a parameter with given index of the last received and recognized command
* Input          : paramIndex - index of the parameter string to return.
* Output         : None
* Return         : Pointer to the parameter string.
*******************************************************************************/
char* UartUserGetParam(int paramIndex)
{
	if( (currentCommand != MP3_COMMAND_NONE) && (paramIndex < PARAMS_MAX_NUM) )
		return commandParams[paramIndex];
	else
		return NULL;
}

/*******************************************************************************
* Function Name  : UartUserGetParamsNum(..)
* Description    : Gets number of parameters of the last received and recognized command
* Input          : None
* Output         : None
* Return         : The number of parameters of the last received and recognized command.
*******************************************************************************/
int UartUserGetParamsNum(void)
{
	int i;
	
	if(currentCommand != MP3_COMMAND_NONE) {
		for(i = 0; (i < PARAMS_MAX_NUM) && commandParams[i][0] != '\0'; i++);
		
		return i;
	}
	else
		return 0;
}

/*******************************************************************************
* Function Name  : UartUserAppendMsg(..)
* Description    : Appends string to the message to send over the UART, if no more space is available just returns
* Input          : msg - pointer to the string to append to the message
* Output         : None
* Return         : TRUE if successfully appended, FALSE if error occurred
*******************************************************************************/
Boolean UartUserAppendMsg(char *msg)
{
	Boolean status = FALSE;
	
	if(!txBufferLocked) {
		strncpy(txBuffer + txBufferIndex, (const char *)msg, TX_BUFFER_LENGTH - txBufferIndex - 1);
		txBuffer[TX_BUFFER_LENGTH - 1] = '\0';
		txBufferIndex = strlen(txBuffer);
		
		status = TRUE;
	}
	
	return status;
}

/*******************************************************************************
* Function Name  : UartUserResetMsg(..)
* Description    : Deletes all appended strings if message sending has not been started yet
* Input          : None
* Output         : None
* Return         : TRUE if successfully reset, FALSE if error occurred
*******************************************************************************/
Boolean UartUserResetMsg(void)
{
	if(!txBufferLocked) {
		txBufferIndex = 0;
		return TRUE;
	}
	
	return FALSE;
}

/*******************************************************************************
* Function Name  : UartUserSendMsg(..)
* Description    : Starts sending message over UART, if any strings have been appended
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void UartUserSendMsg(void)
{
	if(!txBufferLocked) {
		if(txBufferIndex > 0) {
			txBufferLocked = TRUE;
			txBufferIndex = 0;
		}
	}
}

/*******************************************************************************
* Function Name  : UartUserGenerateCommand(..)
* Description    : Generates a user command as if received through the UART
* *NOTE: do not call from interrupts!!
* Input          :	command - the command to generate
*				pParams - pointer to the array of parameters to append (place NULL if no params needed)
*				numberOfParams - if pParams is not NULL this indicates the number of parameters to append
* Output         : None
* Return         : TRUE on success, FALSE if error occured
*******************************************************************************/
Boolean UartUserGenerateCommand(MP3_PLAYER_UART_COMMAND command, char **pParams, int numberOfParams)
{	
	if(pParams != NULL) {
		if(numberOfParams > PARAMS_MAX_NUM)
			return FALSE;
		
		for(int i = 0; i < numberOfParams; i++)
			strncpy(commandParams[i], *(pParams + i), PARAM_CHAR_LENGTH - 1);
	}
	
	currentCommand = command;
	
	return TRUE;
}
			
/* Interrupt handlers */

/*******************************************************************************
* Function Name  : UartUserDataReveived(..)
* Description    : This function is triggered when a character is received over UART (called in 'stm32f10x_it.c')
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void UartUserDataReveived(void)
{
	char dummy;
	
	// if overflow detected discard all data and start filling again
	if(rxBufferIndex >= RX_BUFFER_LENGTH) {
		rxBufferIndex = 0;
	}
	
	// if buffer already contains command candidate do not accept new characters
	if(rxBufferLocked) {
		dummy = (char)USART_ReceiveData(USART2);
		return;
	}

	// Read one byte from the receive data register
	rxBuffer[rxBufferIndex] = (char)USART_ReceiveData(USART2);
	
	// if newline character received then process command
	if( (rxBuffer[rxBufferIndex] == '\n') || rxBuffer[rxBufferIndex] == '\r') {
		rxBuffer[rxBufferIndex] = '\n';
		rxBuffer[rxBufferIndex + 1] = '\0'; // terminate string
		rxBufferLocked = TRUE;
		rxBufferIndex = 0;
	}
	else {
		rxBufferIndex++;
	}
        dummy++;  // to avoid warning
}

/*******************************************************************************
* Function Name  : UartUserDataSent(..)
* Description    : This function is triggered when a character is sent over UART (called in 'stm32f10x_it.c')
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void UartUserDataSent(void)
{
	// if end of string is detected don't send anything more
	if(txBuffer[txBufferIndex] == '\0') {
		txBufferIndex = 0;
		txBufferLocked = FALSE;

		/* Disable the USART2 Transmit interrupt */
		USART_ITConfig(USART2, USART_IT_TXE, DISABLE);
	}
	else {
		/* Write one byte to the transmit data register */
		USART_SendData(USART2, txBuffer[txBufferIndex++]);
	}
}

